#!/usr/bin/perl -w

# Wür weniger Fehler durch Tippfehler
use strict;
use warnings;

# Einheitliche Datenbankschnittstelle einbinden
use DBI;

# Konstanten (anpassbar)
#
# Logdatei
my $logdatei = "/home/heitmann/kluedo_urn_ergaenzen.log";
# Ausgabedatei (CSV)
my $ausgabedatei = "/home/heitmann/kluedo_urn_ergaenzen.csv";
# Datenbank
my $db_name = "opus413_prod";
my $db_host = "localhost";
my $db_port = "3306";
my $db_user = "opus413user";
my $db_passwort = "passwort";
# URN-ID bei der gestartet wird
my $urn_id = 1;

# Variablen (nicht anpassbar)
my $debug = "false";
my $anzahl_datensaetze_gesamt = 0;
my $anzahl_datensaetze_korrigiert = 0;


# Logdatei öffnen
open(LOG, ">>$logdatei") or die ("Kann Logdatei $logdatei nicht erzeugen.\n");
print(LOG "\n\n");
print(LOG localtime(time)."\n");

# Ausgabedatei öffnen
open(AUSGABE, ">$ausgabedatei") or die ("Kann Ausgabedatei $logdatei nicht erzeugen.\n");
# Kopfzeile einfügen
print(AUSGABE "Dokument-ID;URN\n");	

# Header ausgeben
print("DOKUMENTEN OHNE URN EINE URN IN OPUS V4 ZUWEISEN\n");
print("------------------------------------------------\n\n");
print(localtime(time)."  KORREKTUR START\n\n");

# Datenbankverbindungen herstellen
my $db_connection = DBI->connect("DBI:mysql:database=$db_name;host=$db_host;port=$db_port", $db_user, $db_passwort, {AutoCommit => 1})
  or (print(LOG "FEHLER:       Verbindung zur Datenbank \"$db_name\" kann nicht hergestellt werden.\n") and
  print("FEHLER:      DB-Verbindung fehlgeschlagen. Siehe Logdatei.\n") and die($DBI::errstr));
  
# <Funktionen>
#
# Fachbereich ermitteln
# PARAMETER:  keiner
# RETURN:     URN (String)
sub func_urn_ermitteln() {
    # Variable um berechneten URN mit Prüfziffer zwischenzuspeichern
	my $generated_urn;
	
	# Folgende Variablen müssen vor der WHILE-Schleife deklariert werden
	my $db_statement;
	my $db_stmt_handler;
	
	# Ermitteln des nächsten freien URN:
	# URN generieren und prüfen, ob in DB bereits vorhanden. Falls ja, Schleife
	# bis ein URN gefunden, wird der in DB noch nicht existiert.
	do {
		# URN anhand akuteller URN-ID ermitteln
		$generated_urn = `/usr/bin/php urn_berechnen.php $urn_id`;
		# URN-ID anheben
		$urn_id++;
		
		# Prüfen ob URN bereits existiert
		$db_statement = "SELECT di.document_id FROM document_identifiers di WHERE di.type = 'urn' AND di.value = '".$generated_urn."'";
		$db_stmt_handler = $db_connection->prepare($db_statement) or die($DBI::errstr);
		$db_stmt_handler->execute() or die($DBI::errstr);
	} while (my $ergebnis = $db_stmt_handler->fetchrow_array);
	
	return $generated_urn;
}
#
# URN setzen
# PARAMETER:  $_[0]: Dokument-ID, $_[1]: URN (String)
# RETURN:     keiner
sub func_urn_setzen() {
	# Statement zum setzen des URN
	my $db_statement = "INSERT INTO document_identifiers (document_id, type, value) VALUES (?, 'urn', ?)";
	my $db_stmt_handler = $db_connection->do($db_statement, undef, $_[0], $_[1]) or die($DBI::errstr);
}
# </Funktionen>

# MAIN
#
# DB-Statement erzeugen und ausführen
#
# OPUS4-Dokument-IDs von veröffentlichten Dokumenten ermitteln, welche keinen URN haben
my $db_statement_1 = "SELECT d.id FROM documents d WHERE d.server_state = 'published' AND d.id NOT IN (SELECT DISTINCT di.document_id FROM document_identifiers di WHERE di.type = 'urn')";
my $db_stmt_handler_1 = $db_connection->prepare($db_statement_1) or die($DBI::errstr);
$db_stmt_handler_1->execute() or die($DBI::errstr);

# Ermittelte OPUS4-IDs einzeln durchgehen und Korrektur vornehmen 
while (my $akt_document_id = $db_stmt_handler_1->fetchrow_array) {
	
	# Datensatzzähler erhöhen
	$anzahl_datensaetze_gesamt += 1;
	
	# DEBUG: Ausgabe der ermittelten Werte
	if ($debug eq "true") {
		print("\n");
		print("Dokument-ID:        $akt_document_id\n");
	}
	
	# URN ermitteln
	my $akt_urn = &func_urn_ermitteln($akt_document_id);
	
	# URN setzen
	&func_urn_setzen($akt_document_id, $akt_urn);
	
	# Informationen ausgeben und in CSV sichern
	print("Dokument-ID: ".$akt_document_id.", URN: ".$akt_urn."\n");
	print(AUSGABE $akt_document_id.";".$akt_urn."\n");	

	# Datensatzzähler erhöhen
	$anzahl_datensaetze_korrigiert += 1;
		
	# Leerzeile zwischen Datensätzen ausgeben
	print("\n");
}

# Datensatzzähler ausgeben
print("\n\n");
print("Anzahl der korrigierten Datensaetze\n");
print("---------------------------------\n");
print("Datensätze gesamt:         $anzahl_datensaetze_gesamt\n");
print("Datensätze korrigiert:     $anzahl_datensaetze_korrigiert\n");

# Footer ausgeben
print("\n");
print(localtime(time)."  KORREKTUR ENDE\n");

# Ressourcen freigen
$db_stmt_handler_1->finish();

# Offene Dateien schließen
close(LOG);
close(AUSGABE);